@@ -1,5 +1,5 @@
 /**
- * Copyright © 2019 spypunk <spypunk@gmail.com>
+ * Copyright © 2019-2020 spypunk <spypunk@gmail.com>
  *
  * This work is free. You can redistribute it and/or modify it under the
  * terms of the Do What The Fuck You Want To Public License, Version 2,
@@ -85,7 +85,6 @@ class SpongeCommand : CliktCommand(name = "sponge", printHelpOnEmptyArgs = true)
         }
 
         try {
-            val spongeService = SpongeService()
             val spongeInput = SpongeInput(
                 spongeUri,
                 outputDirectory,
@@ -96,7 +95,7 @@ class SpongeCommand : CliktCommand(name = "sponge", printHelpOnEmptyArgs = true)
                 concurrentRequests,
                 concurrentDownloads)
 
-            Sponge(spongeService, spongeInput).execute()
+            Sponge(SpongeService(), spongeInput).execute()
         } catch (t: Throwable) {
             System.err.println("Unexpected error encountered: : ${t.rootMessage()}")
 
