@@ -1,5 +1,5 @@
 /**
- * Copyright © 2019 spypunk <spypunk@gmail.com>
+ * Copyright © 2019-2020 spypunk <spypunk@gmail.com>
  *
  * This work is free. You can redistribute it and/or modify it under the
  * terms of the Do What The Fuck You Want To Public License, Version 2,
@@ -23,11 +23,11 @@ private const val REQUEST_TIMEOUT = 30_000
 private const val ENCODING = "gzip, deflate"
 private const val REFERRER = "https://www.google.com"
 private const val USER_AGENT = "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) " +
-    "Chrome/78.0.3904.97 Safari/537.36"
+    "Chrome/79.0.3945.88 Safari/537.36"
 
 class SpongeService {
     fun request(spongeUri: SpongeUri, timeout: Int = REQUEST_TIMEOUT): Connection.Response {
-        val connection = Jsoup.connect(spongeUri.toString())
+        return Jsoup.connect(spongeUri.toString())
             .timeout(timeout)
             .header(HttpHeaders.ACCEPT_ENCODING, ENCODING)
             .referrer(REFERRER)
@@ -36,12 +36,11 @@ class SpongeService {
             .ignoreHttpErrors(true)
             .ignoreContentType(true)
             .followRedirects(true)
-
-        return retry(IOException::class) { connection.execute() }
+            .let { retry(IOException::class) { it.execute() } }
     }
 
     fun download(spongeUri: SpongeUri, path: Path) {
-        Files.createDirectories(path)
+        Files.createDirectories(path.parent)
 
         request(spongeUri, 0).bodyStream()
             .use { Files.copy(it, path, StandardCopyOption.REPLACE_EXISTING) }
@@ -56,9 +55,7 @@ class SpongeService {
             try {
                 return block.invoke()
             } catch (e: Exception) {
-                if (!clazz.isSuperclassOf(e::class)) {
-                    throw e
-                }
+                if (!clazz.isSuperclassOf(e::class)) throw e
 
                 exception = e
             }
