@@ -1,5 +1,5 @@
 /**
- * Copyright © 2019 spypunk <spypunk@gmail.com>
+ * Copyright © 2019-2020 spypunk <spypunk@gmail.com>
  *
  * This work is free. You can redistribute it and/or modify it under the
  * terms of the Do What The Fuck You Want To Public License, Version 2,
@@ -11,13 +11,13 @@ package spypunk.sponge
 import io.mockk.every
 import io.mockk.mockk
 import io.mockk.verify
-import org.apache.commons.io.FileUtils
 import org.apache.commons.io.FilenameUtils
 import org.apache.http.entity.ContentType
 import org.jsoup.Connection
 import org.junit.jupiter.api.BeforeEach
 import org.junit.jupiter.api.Test
 import java.io.IOException
+import java.nio.file.Files
 import java.nio.file.Path
 
 class SpongeTest {
@@ -38,7 +38,7 @@ class SpongeTest {
 
     @BeforeEach
     fun beforeEach() {
-        FileUtils.deleteDirectory(outputDirectory.toFile())
+        Files.deleteIfExists(outputDirectory)
     }
 
     @Test
@@ -47,7 +47,7 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.download(any(), any()) }
     }
 
@@ -57,7 +57,7 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.download(any(), any()) }
     }
 
@@ -80,9 +80,9 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
-        verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
+        verify { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(fileUri) }
+        verify { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
 
     @Test
@@ -107,11 +107,11 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
-        verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
-        verify(exactly = 1) { spongeService.request(imageFileUri) }
-        verify(exactly = 1) { spongeService.download(imageFileUri, getOutputFilePath(imageFileUri)) }
+        verify { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(fileUri) }
+        verify { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
+        verify(exactly = 0) { spongeService.request(imageFileUri) }
+        verify { spongeService.download(imageFileUri, getOutputFilePath(imageFileUri)) }
     }
 
     @Test
@@ -133,7 +133,7 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.request(fileUri) }
         verify(exactly = 0) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
@@ -157,14 +157,37 @@ class SpongeTest {
 
         executeSponge(spongeInputWithSubdomains)
 
-        verify(exactly = 1) { spongeService.request(spongeInputWithSubdomains.spongeUri) }
-        verify(exactly = 1) { spongeService.request(fileUri) }
+        verify { spongeService.request(spongeInputWithSubdomains.spongeUri) }
+        verify { spongeService.request(fileUri) }
 
-        verify(exactly = 1) {
+        verify {
             spongeService.download(fileUri, getOutputFilePath(fileUri))
         }
     }
 
+    @Test
+    fun testDocumentWithIgnoredLinkAndSubdomainEnabled() {
+        val fileUri = "https://test.test2.com/$fileName".toSpongeUri()
+
+        givenDocument(
+            spongeInputWithSubdomains.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$fileUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenFile(fileUri)
+
+        executeSponge(spongeInputWithSubdomains)
+
+        verify { spongeService.request(spongeInputWithSubdomains.spongeUri) }
+        verify(exactly = 0) { spongeService.request(fileUri) }
+    }
+
     @Test
     fun testDocumentWithChildDocumentAndLink() {
         val childDocumentUri = "https://test.com/test".toSpongeUri()
@@ -196,11 +219,142 @@ class SpongeTest {
 
         executeSponge(spongeInputWithDepthTwo)
 
-        verify(exactly = 1) { spongeService.request(spongeInputWithDepthTwo.spongeUri) }
-        verify(exactly = 1) { spongeService.request(childDocumentUri) }
-        verify(exactly = 1) { spongeService.request(fileUri) }
+        verify { spongeService.request(spongeInputWithDepthTwo.spongeUri) }
+        verify { spongeService.request(childDocumentUri) }
+        verify { spongeService.request(fileUri) }
 
-        verify(exactly = 1) {
+        verify {
+            spongeService.download(fileUri, getOutputFilePath(fileUri))
+        }
+    }
+
+    @Test
+    fun testDocumentWithChildDocumentAndDuplicateLink() {
+        val childDocumentUri = "https://test.com/test".toSpongeUri()
+        val fileUri = "${spongeInputWithDepthTwo.spongeUri}/$fileName".toSpongeUri()
+
+        givenDocument(
+            spongeInputWithDepthTwo.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$fileUri" />
+                            <a href="$childDocumentUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenDocument(
+            childDocumentUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$fileUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenFile(fileUri)
+
+        executeSponge(spongeInputWithDepthTwo)
+
+        verify { spongeService.request(spongeInputWithDepthTwo.spongeUri) }
+        verify { spongeService.request(childDocumentUri) }
+        verify { spongeService.request(fileUri) }
+
+        verify {
+            spongeService.download(fileUri, getOutputFilePath(fileUri))
+        }
+    }
+
+    @Test
+    fun testDocumentWithChildDocumentEqualsToOneParent() {
+        val childDocumentUri = "https://test.com/test".toSpongeUri()
+
+        givenDocument(
+            spongeInputWithDepthTwo.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$childDocumentUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenDocument(
+            childDocumentUri,
+            """
+                    <html>
+                        <body>
+                           <a href="${spongeInputWithDepthTwo.spongeUri}" />
+                        </body>
+                    </html>
+                """
+        )
+
+        executeSponge(spongeInputWithDepthTwo)
+
+        verify { spongeService.request(spongeInputWithDepthTwo.spongeUri) }
+        verify { spongeService.request(childDocumentUri) }
+    }
+
+    @Test
+    fun testDocumentWithChildDocumentEqualsToDirectParent() {
+        givenDocument(
+            spongeInput.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="${spongeInput.spongeUri}" />
+                        </body>
+                    </html>
+                """
+        )
+
+        executeSponge(spongeInput)
+
+        verify { spongeService.request(spongeInput.spongeUri) }
+    }
+
+    @Test
+    fun testDocumentWithTooDeepChildDocumentAndLink() {
+        val childDocumentUri = "https://test.com/test".toSpongeUri()
+        val fileUri = "${spongeInput.spongeUri}/$fileName".toSpongeUri()
+
+        givenDocument(
+            spongeInput.spongeUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$childDocumentUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenDocument(
+            childDocumentUri,
+            """
+                    <html>
+                        <body>
+                            <a href="$fileUri" />
+                        </body>
+                    </html>
+                """
+        )
+
+        givenFile(fileUri)
+
+        executeSponge(spongeInput)
+
+        verify { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(childDocumentUri) }
+        verify(exactly = 0) { spongeService.request(fileUri) }
+
+        verify(exactly = 0) {
             spongeService.download(fileUri, getOutputFilePath(fileUri))
         }
     }
@@ -222,7 +376,7 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(spongeInput.spongeUri) }
         verify(exactly = 0) { spongeService.request(childDocumentUri) }
     }
 
@@ -243,8 +397,8 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
-        verify(exactly = 1) { spongeService.request(any()) }
+        verify { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(any()) }
     }
 
     @Test
@@ -271,15 +425,15 @@ class SpongeTest {
 
         executeSponge(spongeInput)
 
-        verify(exactly = 1) { spongeService.request(spongeInput.spongeUri) }
-        verify(exactly = 1) { spongeService.request(failingFileUri) }
+        verify { spongeService.request(spongeInput.spongeUri) }
+        verify { spongeService.request(failingFileUri) }
 
         verify(exactly = 0) {
             spongeService.download(failingFileUri, getOutputFilePath(failingFileUri))
         }
 
-        verify(exactly = 1) { spongeService.request(fileUri) }
-        verify(exactly = 1) { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
+        verify { spongeService.request(fileUri) }
+        verify { spongeService.download(fileUri, getOutputFilePath(fileUri)) }
     }
 
     private fun getOutputFilePath(spongeUri: SpongeUri): Path {
